# -*- coding: utf-8 -*-

import csv
import json
from makeunicodedata import *


reader = csv.reader(open('UnicodeData.txt', 'r'), delimiter=';')

# ECMAScript 5 $ 7.2
whitespace = [0x9, 0xb, 0xc, 0x20, 0xa0, 0xfeff]
# $ 7.3
line_terminator = [0xa, 0xd, 0x2028, 0x2029]


# These are also part of IdentifierPart $7.6
ZWNJ = 0x200C
ZWJ = 0x200D


FLAG_WHITESPACE = 1 << 0
FLAG_LETTER = 1 << 1
FLAG_IDENTIFIER_PART = 1 << 2
FLAG_NO_DELTA = 1 << 3
FLAG_ENCLOSING_MARK = 1 << 4
FLAG_COMBINING_SPACING_MARK = 1 << 5

MAX = 0xffff


dummy = (0, 0, 0)
table = [dummy]
cache = {0: dummy}
index = [0] * (MAX + 1) #after that all is 0

test = open('test', 'w')

out = ""
for row in reader:
    code_point = row[0]
    category = row[2]
    uppercase = row[-3]
    lowercase = row[-2]

    code = int(code_point, 16)
    flags = 0
    
    if code > MAX:
        break
    
    # we use all this in one class, because in practice we don't need to seperate them often
    # 0x85 NEL (Next Line) is not enforced by ECMA-262, but it says:
    """
    Only the characters in Table 3 are treated as line terminators. 
    Other new line or line breaking characters are treated as white space but not as line terminators. 
    The character sequence <CR><LF> is commonly used as a line terminator. 
    It should be considered a single character for the purpose of reporting line numbers.
    """
    
    if category == 'Zs' or code in whitespace or code in line_terminator or code == 0x85:
		flags |= FLAG_WHITESPACE                
    if category in ['Lu', 'Ll', 'Lt', 'Lm', 'Lo', 'Nl']: # $ 7.6 (UnicodeLetter)
        flags |= FLAG_LETTER
    if category in ['Mn', 'Mc', 'Nd', 'Pc'] or code == ZWNJ or code == ZWJ: # $ 7.6 (IdentifierPart)
        flags |= FLAG_IDENTIFIER_PART
    if category == 'Me': # ENCLOSING_MARK 
        flags |= FLAG_ENCLOSING_MARK
    if category == 'Mc': # COMBINING_SPACING_MARK
        flags |= FLAG_COMBINING_SPACING_MARK
    
    if uppercase:
        upper = int(uppercase, 16)
    else:
        upper = code
    
    if lowercase:
        lower = int(lowercase, 16)
    else:
        lower = code
    
    out = out + '    [' + hex(upper) + ', ' + hex(lower) + '], /* ' + hex(code) + ' */ \n'
    
    up_d = upper - code
    low_d = lower - code
    
    if -32768 <= up_d <= 32767 and -32768 <= low_d <= 32767:
        upper = up_d & 0xffff
        lower = low_d & 0xffff
    else:
        flags |= FLAG_NO_DELTA
    
    item = (upper, lower, flags)
    
    i = cache.get(item)
    if i is None:
        cache[item] = i = len(table)
        table.append(item)
    index[code] = i

test.write(out)
print table[index[0xffee]]

index1, index2, shift = splitbins(index, 1)


idx = index1[0xffff >> shift]
print idx

print (idx << shift) + (0xffff & 0x1f)
idx = index2[(idx << shift) + (0xffff & 0x1f)]
print idx

print table[idx]

# verify correctness
for char in index:
    test = table[index[char]]
    
    idx = index1[char >> shift]
    idx = index2[(idx << shift) + (char & ((1 << shift) - 1))]
    
    assert test == table[idx]

out = open('out', 'w')

out.write('CharacterInfo js_charinfo[] = {\n')
for d in table:
    out.write('    {')
    out.write(', '.join((str(e) for e in d)) + '},\n')
out.write('};\n')

def dump(data, name, file):
    out.write('const uint16 ' + name + '[] = {\n')

    s = '   '    
    for i in data:
        s += str(i) + ', '
    s = s[:-2]
    file.write(s)
    file.write('\n};\n')

dump(index1, 'index1', out)
dump(index2, 'index2', out)
